from qpd.config import Config
from qpd.networks.network_interface import NetworkInterface
import torch
import torch.nn as nn
from torch.distributions import Normal

class StudentSixModel(NetworkInterface):
    def __init__(self, config: Config):
        super(StudentSixModel, self).__init__(config)

        self.net = nn.Sequential(
            nn.Linear(self.config.observation_shape[0], 64),
            nn.ReLU(),

            nn.Linear(64, 64),
            nn.ReLU(),

            nn.Linear(64, 64),
            nn.ReLU()
        )

        self.action_head = nn.Linear(64, 6)
        self.std_head = nn.Linear(64, 6)
        self.critic_head = nn.Linear(64, 1)

        for module in self.children():
            if type(module) == nn.Linear:
                module.bias.data.uniform_(0.0) # type: ignore
                module.weight.data.uniform_(0, 0.01) # type: ignore

    def forward(self, observations):
        x = self.net(observations)
        mean_actions = self.action_head(x)
        std = self.std_head(x)**2
        dist = Normal(mean_actions, std)
        self.actions = dist.sample()

        value = self.critic_head(x)

        value = torch.cat([value, value], dim=0)

        return mean_actions, value, std
    
    def get_actions(self):
        return self.actions.cpu().numpy()