from qpd.config import Config
from qpd.networks.network_interface import NetworkInterface
import torch
import torch.nn as nn
from torch.distributions import Normal

class TinyStudentDQN(NetworkInterface):
    def __init__(self, config: Config):
        super(TinyStudentDQN, self).__init__(config)

        self.net = nn.Sequential(
            nn.Linear(self.config.observation_shape[0], 16),
            nn.ReLU(),

            nn.Linear(16, 16),
            nn.ReLU(),

            # nn.Linear(16, 16),
            # nn.ReLU()
        )

        self.action_head = nn.Linear(16, 2)

        for module in self.children():
            if type(module) == nn.Linear:
                module.bias.data.uniform_(0.0) # type: ignore
                module.weight.data.uniform_(0, 0.01) # type: ignore

    def forward(self, observations):
        x = self.net(observations)
        self.mean_actions = self.action_head(x)

        return self.mean_actions, None, None
    
    def get_actions(self):
        actions = self.mean_actions.argmax(dim=1).reshape(-1)
        return actions.cpu().numpy()