import torch
import torch.nn as nn

import qpd.config as config

class NetworkInterface(nn.Module):
    def __init__(self, config: "config.Config"):
        super(NetworkInterface, self).__init__()
        self.config: "config.Config" = config

    def forward(self, x):
        raise NotImplementedError(f"Forward method not implemented for {type(self)}!")

    def get_actions(self):
        raise NotImplementedError(f"Actions method not implemented for {type(self)}!")
    
    def save(self, path):
        torch.save(self.state_dict(), path + ".pth")

    def load(self, path):
        self.load_state_dict(torch.load(path))
        self.eval()
    
    def observation_to_tensor(self, x):
        return torch.from_numpy(x)