from typing import Type, Union

import gym
import numpy as np
import torch

import qpd.networks.wrapper.stable_baselines.student as student
import qpd.networks.wrapper.model_wrapper as model_wrapper
from stable_baselines3.common.base_class import BaseAlgorithm
from stable_baselines3.common.policies import BasePolicy

class SBWrapper(model_wrapper.ModelWrapper):
    algorithm_type: Type[BaseAlgorithm] = None

    def __init__(self, config, model: Union[BaseAlgorithm | BasePolicy]):
        super(SBWrapper, self).__init__(config, model)
        self.policy: BasePolicy = model.policy if isinstance(model, BaseAlgorithm) else model
        assert isinstance(self.policy, BasePolicy), "Incorrect input model type!" 

    def forward(self, x):
        raise NotImplementedError("Not implemented forward method!")
    
    def _get_actions(self):
        raise NotImplementedError("Not implemented _getactions method!")

    def observation_to_tensor(self, x):
        return self.policy.obs_to_tensor(x)[0]
    
    def save(self, path):
        student = self.construct_student()
        student.policy = self.policy

        student.save(path + ".zip")

    def load(self, path):
        self.policy = self.algorithm_type.load(path).policy
    
    def get_actions(self):
        with torch.no_grad():
            actions = self._get_actions()

        actions = actions.cpu().numpy()
        if isinstance(self.policy.action_space, gym.spaces.Box):
            if self.policy.squash_output:
                # Rescale to proper domain when using squashing
                actions = self.policy.unscale_action(actions)
            else:
                # Actions could be on arbitrary scale, so clip the actions to avoid
                # out of bound error (e.g. if sampling from a Gaussian distribution)
                actions = np.clip(actions, self.policy.action_space.low, self.policy.action_space.high)

        return actions

    def construct_student(self):
        tmp = {
            "learning_rate": self.config.student_config.learning_rate,
            "policy_kwargs": {
                "net_arch": [],
                "features_extractor_class": student.SBStudentNet,
                "features_extractor_kwargs": {
                    "network": self.config.student_config.student_network
                },
                "activation_fn": self.config.student_config.activation_func
            }
        }
        tmp.update(self.config.student_config.extra_student_kwargs)
        return self.original_type(type(self.policy), self.config.environment_constructor(), **tmp)