from typing import Type
import torch as th
import torch.nn as nn
from gym import spaces

import qpd.networks.wrapper.student.student_net as student

class CNNStudentNet(student.StudentNet):
    def __init__(self, observation_space: spaces.Box, n_output):
        super(CNNStudentNet, self).__init__(observation_space, n_output)
        # We assume CxHxW images (channels first)
        # Re-ordering will be done by pre-preprocessing or wrapper
        n_input_channels = observation_space.shape[0]

        self.cnn = nn.Sequential(
            nn.Conv2d(n_input_channels, 16, kernel_size=8, stride=4),
            nn.ReLU(),

            nn.Conv2d(16, 16, kernel_size=4, stride=2),
            nn.ReLU(),

            nn.Conv2d(16, 16, kernel_size=3, stride=1),
            nn.ReLU(),
            nn.Flatten()
        )

        # Compute shape by doing one forward pass
        with th.no_grad():
            n_flatten = self.cnn(
                th.as_tensor(observation_space.sample()[None]).float()
            ).shape[1]

        self.linear = nn.Sequential(
            nn.Linear(n_flatten, self.features_dim), 
            nn.ReLU()
        )

    def forward(self, observations: th.Tensor) -> th.Tensor:
        return self.linear(self.cnn(observations)) 