from typing import Type
import torch as th
import torch.nn as nn
from gym import spaces

import qpd.networks.wrapper.student.student_net as student

class FCStudentNet(student.StudentNet):
    def __init__(self, observation_space: spaces.Box, n_output: int):
        super(FCStudentNet, self).__init__(observation_space, 64)

        n_input_channels = observation_space.shape[0]

        self.net = nn.Sequential(
            nn.Linear(n_input_channels, 64),
            nn.ReLU(),

            nn.Linear(64, 64),
            nn.ReLU(),

            nn.Linear(64, self.features_dim),
            nn.ReLU()
        )

        for module in self.children():
            if type(module) == nn.Linear:
                module.bias.data.uniform_(0.0) # type: ignore
                module.weight.data.uniform_(0, 0.01) # type: ignore

    def forward(self, observations: th.Tensor) -> th.Tensor:
        return self.net(observations)