from distiller.quantization import DorefaQuantizer
from .summary import summary
import torch.nn as nn
import torch
import os


class Flatten(nn.Module):
    def forward(self, x):
        return x.reshape(x.size(0), -1)


def init_module(module, weight_init, bias_init, gain=1):
    weight_init(module.weight.data, gain=gain)
    bias_init(module.bias.data)
    return module


def get_output_size(input_shape, layer):
    sample = torch.zeros(size=(1, *input_shape))
    return layer(sample).view(1, -1).size(1)


def get_network(config, Network, load=None, quantization=0, verbose=True):
    network = Network(config)

    if verbose:
        summary(network)
    if quantization:
        bits = quantization
        optimizer = torch.optim.RMSprop(network.parameters(), lr=0)
        quantizer = DorefaQuantizer(network, optimizer, bits_activations=bits, bits_weights=bits, bits_bias=bits)
        quantizer.prepare_model()
        quantizer.quantize_params()

    if load and (type(load) != str or os.path.exists(load)):
        network = network.load(load)

    network.eval()
    return network

def get_output(action, value, action_std):
    if action_std is not None:
        return torch.cat([action, value, action_std], dim=0)
    if value is not None:
        return torch.cat([action, value], dim=0)
    return action
