import os
from collections import OrderedDict
from distiller.quantization import DorefaQuantizer
import numpy as np
from onnxruntime.quantization import quantize_dynamic
import torch

def filter_state_dict(state_dict: OrderedDict):
    new = OrderedDict()
    for key, state in state_dict.items():
        key_split = key.split(".")
        
        state_type = key_split[-1]

        if state_type == "float_weight" or state_type == "float_bias":
            continue
            
        new[key] = state
    return new


def clean_student(student, config):
    clean_student = type(student)(config)
    clean_student.load_state_dict(filter_state_dict(student.state_dict()))
    return clean_student


def export_student_as_onnx(input, output, student_type, config, get_environment, verbose=True, onnx_input_names=["input_1"], onnx_output_names=["output_1"], opset_version=14, qat=True):
    # Loading quantized student
    student = student_type(config)
    bits = config.quantization_config.bits

    if qat: 
        q = DorefaQuantizer(student, torch.optim.RMSprop(student.parameters(), lr=config.student_config.learning_rate), bits, bits, bits)
        q.prepare_model()

    student.load(input)
    
    c_student = clean_student(student, config)

    env = get_environment(config)
    state = env.reset().astype(np.float32)

    path = output.split("/")[:-1]
    
    orig_onnx = "/".join(path) + "/" + "orig.onnx"
    temp_onnx = "/".join(path) + "/" + "clean.onnx"

    torch.onnx.export(student, torch.from_numpy(state), orig_onnx, verbose=verbose, input_names=onnx_input_names, output_names=onnx_output_names, opset_version=opset_version)
    torch.onnx.export(c_student, torch.from_numpy(state), temp_onnx, verbose=verbose, input_names=onnx_input_names, output_names=onnx_output_names, opset_version=opset_version)
    quantize_dynamic(orig_onnx, output)
    # os.remove(temp_onnx)
