from typing import List, Any, Tuple
import torch

def summary(network):
    print("======================================")
    print("= Network Summary                    =")
    print("======================================")
    print(print_layers([(network.__class__.__name__, extract_parameters(network.parameters()), find_layers(network))]))

def find_layers(network) -> List[Tuple[str, Any]]:
    layers = []
    for child in network.children():
        children = list(child.children())
        if not len(children):
            layers.append((child.__class__.__name__, extract_parameters(child.parameters())))
        else:
            layers.append((child.__class__.__name__, extract_parameters(child.parameters()), find_layers(child)))
    return layers

def extract_parameters(parameters) -> int:
    total = 0
    for param in parameters:
        if isinstance(param, torch.Tensor):
            total += torch.prod(torch.LongTensor(list(param.size())))
        else:
            total += extract_parameters(param)
    return int(total)


def print_layers(layers) -> str:
    result = ""
    for layer in layers:
        result += "- " + layer[0]
        if len(layer) == 2:
            result += f': {layer[1]:n}\n'
        else:
            result += f': {layer[1]:n}\n'
            for line in print_layers(layer[2]).splitlines():
                result += "   %s\n" % line
    return result
